function GetBroadcastAddress([string]$address, [string]$subnetMask) {
    # Convert the ip address strings into an array of bytes
    $addressBytes = $address -split "\." | ForEach { [byte] $_ };
    $subnetMaskBytes = $subnetMask -split "\." | ForEach { [byte] $_ };

    $broadcastBytes = New-Object byte[] 4;
    for($i = 0; $i -lt 4; ++$i) {
        # Invert the the bits of the subnet mask
        # We need to add 256 as the -bnot function yields a signed
        # integer/negative numbers
        $inverted = -bnot [byte] $subnetMaskBytes[$i] + 256;

        # link the address bytes with the inverted subnet mask
        $broadcastBytes[$i] = $addressBytes[$i] -bor $inverted;
    }

    return $broadcastBytes -join ".";
}

function GetMacAddress([string]$address, [string]$delimiter = ":") {
    $arp = arp -a | findstr $address;
    if(!$arp) {
        return $null;
    }

    $arp = $arp.Trim() -split "\s+";
    return $arp[1] -replace "-",$delimiter;
}

function CreateMagicPacket([string]$mac) {
    # Split the mac address at the character : and convert the parts to a byte
    # using the 0x prefix, we make sure that the provided $_ value will be
    # interpreted as hex.
    [byte[]] $macBytes = $mac -split ":" | ForEach { [byte] "0x$_" };

    # Now create an array of 6 repetitions of 0xFF and 16 repetitions of the mac address
    [byte[]] $magicPacket = (,0xFF * 6) + ($macBytes * 16);
    return $magicPacket;
}

function SendMagicPacket([byte[]]$magicPacket, [string]$broadcast, [uint32]$port) {
    $udpClient = New-Object System.Net.Sockets.UdpClient;
    $udpClient.Connect($broadcast, $port);
    $udpClient.Send($magicPacket, $magicPacket.Length);
    $udpClient.Close();
}

function WakeOnLan([string]$mac, [string]$broadcast, [uint32]$port) {
    $packet = CreateMagicPacket $mac;
    SendMagicPacket $packet $broadcast $port;
}


# Specify the target host and the proper subnet mask
$hostAddress = "192.168.1.11";
$subnetMask = "255.255.255.0";

# Retrieve the mac address from the specified host
$macAddress = GetMacAddress $hostAddress;
if(!$macAddress) {
    Write-Host "could not retrieve mac address of host $hostAddress";
    return;
}

# Calculate the broadcast address, create the magic packet and send it
$broadcastAddress = GetBroadcastAddress $hostAddress $subnetMask;
$packet = CreateMagicPacket $macAddress;
SendMagicPacket $packet $broadcastAddress 9;
